@@ -57,7 +57,7 @@ public:
 
     /** Return the point type if the board has different types of points.
         For example, in the geometry used in Blokus Trigon, there are two
-        point types (0=downside triangle, 1=upside trinagle); in a regular
+        point types (0=upward triangle, 1=downward triangle); in a regular
         rectangle, there is only one point type. By convention, 0 is the
         type of the point at (0,0).
         @param x The x coordinate (may be negative and/or outside the board).
@@ -110,7 +110,7 @@ public:
     unsigned get_second_dist_to_edge(Point p) const;
 
 protected:
-    Geometry() = default;
+    Geometry();
 
     /** Initialize.
         Subclasses must call this function in their constructors. */
@@ -164,14 +164,14 @@ inline Geometry<P>::Iterator::Iterator(const Geometry& g)
 template<class P>
 inline const P& Geometry<P>::Iterator::operator*() const
 {
-    LIBBOARDGAME_ASSERT(operator bool());
+    LIBBOARDGAME_ASSERT(*this);
     return *m_p;
 }
 
 template<class P>
 inline const P* Geometry<P>::Iterator::operator->() const
 {
-    LIBBOARDGAME_ASSERT(operator bool());
+    LIBBOARDGAME_ASSERT(*this);
     return m_p;
 }
 
@@ -184,10 +184,15 @@ inline Geometry<P>::Iterator::operator bool() const
 template<class P>
 inline void Geometry<P>::Iterator::operator++()
 {
-    LIBBOARDGAME_ASSERT(operator bool());
+    LIBBOARDGAME_ASSERT(*this);
     ++m_p;
 }
 
+template<class P>
+Geometry<P>::Geometry()
+{
+}
+
 template<class P>
 Geometry<P>::~Geometry()
 {
