@@ -23,8 +23,6 @@ class Failure
 public:
     Failure(const string& response = "");
 
-    ~Failure() throw();
-
     string get_response() const;
 
 private:
@@ -36,10 +34,6 @@ inline Failure::Failure(const string& response)
 {
 }
 
-inline Failure::~Failure() throw()
-{
-}
-
 inline string Failure::get_response() const
 {
     return m_response;
