@@ -9,6 +9,7 @@
 
 #include <algorithm>
 #include <cstddef>
+#include <initializer_list>
 #include <iosfwd>
 #include <limits>
 #include "Assert.h"
@@ -47,6 +48,8 @@ public:
     /** Construct list with a single element. */
     explicit ArrayList(const T& t);
 
+    ArrayList(const initializer_list<T>& l);
+
     T& operator[](I i);
 
     const T& operator[](I i) const;
@@ -96,7 +99,7 @@ public:
         @return @c true if element was not already in list. */
     bool include(const T& t);
 
-    /** Removal of first occurance of value.
+    /** Removal of first occurrence of value.
         Preserves the order of elements.
         @return @c true if value was removed. */
     bool remove(const T& t);
@@ -106,15 +109,13 @@ public:
         with the last element and the list size decremented. */
     void remove_fast(iterator i);
 
-    /** Fast removal of first occurance of value.
+    /** Fast removal of first occurrence of value.
         Does not preserve the order of elements. If the value is found,
         it will be replaced with the last element and the list size
         decremented.
         @return @c true if value was removed. */
     bool remove_fast(const T& t);
 
-    bool is_permutation(const ArrayList& l) const;
-
 private:
     I m_size;
 
@@ -133,6 +134,14 @@ inline ArrayList<T,M,I>::ArrayList(const T& t)
     assign(t);
 }
 
+template<typename T, unsigned M, typename I>
+ArrayList<T, M, I>::ArrayList(const initializer_list<T>& l)
+    : m_size(0)
+{
+    for (auto& t : l)
+        push_back(t);
+}
+
 template<typename T, unsigned M, typename I>
 inline T& ArrayList<T,M,I>::operator[](I i)
 {
@@ -259,18 +268,6 @@ bool ArrayList<T,M,I>::include(const T& t)
     return true;
 }
 
-template<typename T, unsigned M, typename I>
-bool ArrayList<T,M,I>::is_permutation(const ArrayList& l) const
-{
-    if (size() != l.size())
-        return false;
-    ArrayList sorted_this(*this);
-    sort(sorted_this.begin(), sorted_this.end());
-    ArrayList sorted_other(l);
-    sort(sorted_other.begin(), sorted_other.end());
-    return equal(sorted_this.begin(), sorted_this.end(), sorted_other.begin());
-}
-
 template<typename T, unsigned M, typename I>
 inline const T& ArrayList<T,M,I>::pop_back()
 {
