@@ -91,7 +91,15 @@ void Board::gen_moves(Color c, ArrayList<Move, Move::range>& moves) const
             if (is_attach_point(*i, c) && ! m_state_color[c].forbidden[*i])
                 gen_moves(c, *i, get_adj_status(*i, c), m_marker, moves);
     }
-    m_marker.clear_all_set_known(moves);
+    m_marker.clear(moves);
+}
+
+void Board::gen_moves(Color c, Point p,
+                      ArrayList<Move, Move::range>& moves) const
+{
+    moves.clear();
+    gen_moves(c, p, m_marker, moves);
+    m_marker.clear(moves);
 }
 
 void Board::gen_moves(Color c, Point p, MoveMarker& marker,
@@ -231,7 +239,6 @@ void Board::init(Variant variant, const Setup* setup)
         m_state_color[*i].pieces_left.clear();
         m_state_color[*i].nu_onboard_pieces = 0;
         m_state_color[*i].points = 0;
-        m_state_color[*i].bonus = 0;
         for (Piece::IntType j = 0; j < get_nu_uniq_pieces(); ++j)
             m_state_color[*i].pieces_left.push_back(Piece(j));
         m_state_color[*i].nu_left_piece.fill(m_nu_piece_instances);
@@ -245,14 +252,12 @@ void Board::init(Variant variant, const Setup* setup)
     else
     {
         m_setup = *setup;
-        for (ColorIterator i(m_nu_colors); i; ++i)
-            for (Move mv : setup->placements[*i])
-                place(*i, mv);
+        place_setup(m_setup);
         m_state_base.to_play = setup->to_play;
         optimize_attach_point_lists();
         for (ColorIterator i(m_nu_colors); i; ++i)
             if (m_state_color[*i].pieces_left.empty())
-                m_state_color[*i].bonus = m_bonus_all_pieces;
+                m_state_color[*i].points += m_bonus_all_pieces;
     }
     m_moves.clear();
 }
@@ -341,6 +346,17 @@ void Board::optimize_attach_point_lists()
     }
 }
 
+/** Place setup moves on board.
+    This function is only extracted from init() because it contains the large
+    inline function place(), which will cause a failure to inline and warning
+    with -Winline with GCC 4.8.2. */
+void Board::place_setup(const Setup& setup)
+{
+    for (ColorIterator i(m_nu_colors); i; ++i)
+        for (Move mv : setup.placements[*i])
+            place(*i, mv);
+}
+
 void Board::take_snapshot()
 {
     if (! m_snapshot)
@@ -360,7 +376,7 @@ void Board::take_snapshot()
         snapshot_state.forbidden.memcpy_from(state.forbidden, *m_geo);
         snapshot_state.is_attach_point.memcpy_from(state.is_attach_point,
                                                    *m_geo);
-        // Uncomment one is_trivially_copyable is implemented in GCC and MSVC
+        // Uncomment once is_trivially_copyable is implemented in GCC and MSVC
         //static_assert(is_trivially_copyable<StateColor>::value, "")
         memcpy(&snapshot_state.pieces_left, &state.pieces_left,
                sizeof(StateColor) - offsetof(StateColor, pieces_left));
@@ -379,6 +395,20 @@ void Board::undo()
 
 void Board::write(ostream& out, bool mark_last_move) const
 {
+    // Sort lists of left pieces by name
+    ColorMap<PiecesLeftList> pieces_left;
+    for (ColorIterator i(m_nu_colors); i; ++i)
+    {
+        pieces_left[*i] = m_state_color[*i].pieces_left;
+        sort(pieces_left[*i].begin(), pieces_left[*i].end(),
+             [&](Piece p1, Piece p2)
+             {
+                 return
+                     get_piece_info(p1).get_name()
+                         < get_piece_info(p2).get_name();
+             });
+    }
+
     ColorMove last_mv = ColorMove::null();
     if (mark_last_move)
     {
@@ -397,12 +427,11 @@ void Board::write(ostream& out, bool mark_last_move) const
     auto board_type = get_board_type();
     bool is_trigon = (board_type == BoardType::trigon
                       || board_type == BoardType::trigon_3);
-    write_x_coord(out, width, is_trigon ? 3 : 2);
-    for (unsigned y = height - 1; ; --y)
+    for (unsigned y = 0; y < height; ++y)
     {
-        if (y < 9)
+        if (height - y < 10)
             out << ' ';
-        out << (y + 1) << ' ';
+        out << (height - y) << ' ';
         for (unsigned x = 0; x < width; ++x)
         {
             Point p(x, y, width);
@@ -438,7 +467,7 @@ void Board::write(ostream& out, bool mark_last_move) const
                 else if (is_trigon)
                 {
                     set_color(out, "\x1B[1;30;47m");
-                    out << (m_geo->get_point_type(x, y) == 0 ? '\\' : '/');
+                    out << (m_geo->get_point_type(x, y) == 1 ? '\\' : '/');
                 }
                 else
                 {
@@ -451,7 +480,7 @@ void Board::write(ostream& out, bool mark_last_move) const
                 if (is_trigon && x > 0 && is_onboard(p.get_left()))
                 {
                     set_color(out, "\x1B[1;30;47m");
-                    out << (m_geo->get_point_type(p) == 0 ? '\\' : '/');
+                    out << (m_geo->get_point_type(p) == 1 ? '\\' : '/');
                 }
                 else
                 {
@@ -494,7 +523,7 @@ void Board::write(ostream& out, bool mark_last_move) const
             if (is_onboard(Point(width - 1, y, width)))
             {
                 set_color(out, "\x1B[1;30;47m");
-                out << (m_geo->get_point_type(width - 1, y) != 0 ? '\\' : '/');
+                out << (m_geo->get_point_type(width - 1, y) != 1 ? '\\' : '/');
             }
             else
             {
@@ -503,28 +532,19 @@ void Board::write(ostream& out, bool mark_last_move) const
             }
         }
         set_color(out, "\x1B[0m");
-        out << ' ' << (y + 1);
         if (is_info_location_right)
-        {
-            if (y < 9)
-                out << "   ";
-            else
-                out << "  ";
-            write_info_line(out, height - y - 1);
-        }
+            write_info_line(out, y, pieces_left);
         out << '\n';
-        if (y == 0)
-            break;
     }
     write_x_coord(out, width, is_trigon ? 3 : 2);
     if (! is_info_location_right)
         for (ColorIterator i(m_nu_colors); i; ++i)
         {
             write_color_info_line1(out, *i);
+            out << "  ";
+            write_color_info_line2(out, *i, pieces_left[*i]);
             out << ' ';
-            write_color_info_line2(out, *i);
-            out << ' ';
-            write_color_info_line3(out, *i);
+            write_color_info_line3(out, *i, pieces_left[*i]);
             out << '\n';
         }
 }
@@ -532,68 +552,107 @@ void Board::write(ostream& out, bool mark_last_move) const
 void Board::write_color_info_line1(ostream& out, Color c) const
 {
     set_color(out, m_color_esc_sequence_text[c]);
-    if (get_to_play() == c)
-        out << '*';
+    if (! is_game_over() && get_effective_to_play() == c)
+        out << '(' << (get_nu_moves() + 1) << ") ";
     out << m_color_name[c] << "(" << m_color_char[c] << "): " << get_points(c);
-    unsigned bonus = get_bonus(c);
-    if (bonus > 0)
-        out << " (+" << bonus << ')';
+    if (! has_moves(c))
+        out << '!';
     set_color(out, "\x1B[0m");
 }
 
-void Board::write_color_info_line2(ostream& out, Color c) const
+void Board::write_color_info_line2(ostream& out, Color c,
+                                   const PiecesLeftList& pieces_left) const
 {
     if (m_variant == Variant::junior)
-        write_pieces_left(out, c, 0, 6);
+        write_pieces_left(out, c, pieces_left, 0, 6);
     else
-        write_pieces_left(out, c, 0, 10);
+        write_pieces_left(out, c, pieces_left, 0, 10);
 }
 
-void Board::write_color_info_line3(ostream& out, Color c) const
+void Board::write_color_info_line3(ostream& out, Color c,
+                                   const PiecesLeftList& pieces_left) const
 {
     if (m_variant == Variant::junior)
-        write_pieces_left(out, c, 6, get_nu_uniq_pieces());
+        write_pieces_left(out, c, pieces_left, 6, get_nu_uniq_pieces());
     else
-        write_pieces_left(out, c, 10, get_nu_uniq_pieces());
+        write_pieces_left(out, c, pieces_left, 10, get_nu_uniq_pieces());
 }
 
-void Board::write_info_line(ostream& out, unsigned y) const
+void Board::write_info_line(ostream& out, unsigned y,
+                            const ColorMap<PiecesLeftList>& pieces_left) const
 {
     if (y == 0)
+    {
+        out << "  ";
         write_color_info_line1(out, Color(0));
+    }
     else if (y == 1)
-        write_color_info_line2(out, Color(0));
+    {
+        out << "  ";
+        write_color_info_line2(out, Color(0), pieces_left[Color(0)]);
+    }
     else if (y == 2)
-        write_color_info_line3(out, Color(0));
+    {
+        out << "  ";
+        write_color_info_line3(out, Color(0), pieces_left[Color(0)]);
+    }
     else if (y == 4)
+    {
+        out << "  ";
         write_color_info_line1(out, Color(1));
+    }
     else if (y == 5)
-        write_color_info_line2(out, Color(1));
+    {
+        out << "  ";
+        write_color_info_line2(out, Color(1), pieces_left[Color(1)]);
+    }
     else if (y == 6)
-        write_color_info_line3(out, Color(1));
+    {
+        out << "  ";
+        write_color_info_line3(out, Color(1), pieces_left[Color(1)]);
+    }
     else if (y == 8 && m_nu_colors > 2)
+    {
+        out << "  ";
         write_color_info_line1(out, Color(2));
+    }
     else if (y == 9 && m_nu_colors > 2)
-        write_color_info_line2(out, Color(2));
+    {
+        out << "  ";
+        write_color_info_line2(out, Color(2), pieces_left[Color(2)]);
+    }
     else if (y == 10 && m_nu_colors > 2)
-        write_color_info_line3(out, Color(2));
+    {
+        out << "  ";
+        write_color_info_line3(out, Color(2), pieces_left[Color(2)]);
+    }
     else if (y == 12 && m_nu_colors > 3)
+    {
+        out << "  ";
         write_color_info_line1(out, Color(3));
+    }
     else if (y == 13 && m_nu_colors > 3)
-        write_color_info_line2(out, Color(3));
+    {
+        out << "  ";
+        write_color_info_line2(out, Color(3), pieces_left[Color(3)]);
+    }
     else if (y == 14 && m_nu_colors > 3)
-        write_color_info_line3(out, Color(3));
+    {
+        out << "  ";
+        write_color_info_line3(out, Color(3), pieces_left[Color(3)]);
+    }
 }
 
-void Board::write_pieces_left(ostream& out, Color c, unsigned begin,
-                              unsigned end) const
+void Board::write_pieces_left(ostream& out, Color c,
+                              const PiecesLeftList& pieces_left,
+                              unsigned begin, unsigned end) const
 {
     for (unsigned i = begin; i < end; ++i)
-        if (i < m_state_color[c].pieces_left.size())
+        if (i < pieces_left.size())
         {
             if (i > begin)
                 out << ' ';
-            Piece piece = m_state_color[c].pieces_left[i];
+            Piece piece = pieces_left[i];
             auto& name = get_piece_info(piece).get_name();
             unsigned nu_left = m_state_color[c].nu_left_piece[piece];
             for (unsigned j = 0; j < nu_left; ++j)
