@@ -87,6 +87,10 @@ public:
 
     Color::IntType get_nu_colors() const;
 
+    /** Number of colors that are not played alternately.
+        This is equal to get_nu_colors() apart from Variant::classic_3. */
+    Color::IntType get_nu_nonalt_colors() const;
+
     unsigned get_nu_players() const;
 
 
@@ -124,6 +128,10 @@ public:
         move). */
     Color get_to_play() const;
 
+    /** Get the player who plays the next move for the 4th color in
+        Variant::classic_3. */
+    Color::IntType get_alt_player() const;
+
     /** Get next color to play that still has moves.
         Colors are tried in their playing order starting with get_to_play().
         If no color has moves left, get_to_play() is returned. */
@@ -142,14 +150,9 @@ public:
         of a unique piece per player. */
     unsigned get_nu_left_piece(Color c, Piece piece) const;
 
-    /** Get number of points on board occupied by a color. */
+    /** Get number of points by a color including the bonus. */
     unsigned get_points(Color c) const;
 
-    unsigned get_bonus(Color c) const;
-
-    /** Equivalent to get_points(c) + get_bonus(c) */
-    unsigned get_points_with_bonus(Color c) const;
-
     /** Is a point a potential attachment point for a color.
         Does not check if the point is forbidden. */
     bool is_attach_point(Point p, Color c) const;
@@ -235,6 +238,9 @@ public:
         The generated moves do not include the pass move. */
     void gen_moves(Color c, ArrayList<Move, Move::range>& moves) const;
 
+    void gen_moves(Color c, Point p,
+                   ArrayList<Move, Move::range>& moves) const;
+
     void gen_moves(Color c, Point p, MoveMarker& marker,
                    ArrayList<Move, Move::range>& moves) const;
 
@@ -320,7 +326,8 @@ public:
     /** Get the second color in game variants in which a player plays two
         colors.
         @return The second color of the player that plays color c, or c if
-        the player plays only one color in the current game variant. */
+        the player plays only one color in the current game variant or
+        if the game variant is classic_3. */
     Color get_second_color(Color c) const;
 
     bool is_same_player(Color c1, Color c2) const;
@@ -370,8 +377,6 @@ private:
         unsigned nu_onboard_pieces;
 
         unsigned points;
-
-        unsigned bonus;
     };
 
     /** Snapshot for fast restoration of a previous position.
@@ -456,16 +461,22 @@ private:
 
     void place(Color c, Move mv);
 
-    void write_pieces_left(ostream& out, Color c, unsigned begin,
+    void place_setup(const Setup& setup);
+
+    void write_pieces_left(ostream& out, Color c,
+                           const PiecesLeftList& pieces_left, unsigned begin,
                            unsigned end) const;
 
     void write_color_info_line1(ostream& out, Color c) const;
 
-    void write_color_info_line2(ostream& out, Color c) const;
+    void write_color_info_line2(ostream& out, Color c,
+                                const PiecesLeftList& pieces_left) const;
 
-    void write_color_info_line3(ostream& out, Color c) const;
+    void write_color_info_line3(ostream& out, Color c,
+                                const PiecesLeftList& pieces_left) const;
 
-    void write_info_line(ostream& out, unsigned y) const;
+    void write_info_line(ostream& out, unsigned y,
+                         const ColorMap<PiecesLeftList>& pieces_left) const;
 };
 
 typedef Board::Iterator BoardIterator;
@@ -499,6 +510,12 @@ inline unsigned Board::get_adj_status(Point p, Color c) const
     return result;
 }
 
+inline Color::IntType Board::get_alt_player() const
+{
+    LIBBOARDGAME_ASSERT(m_variant == Variant::classic_3);
+    return static_cast<Color::IntType>(get_nu_onboard_pieces(Color(3)) % 3);
+}
+
 inline const PointList&  Board::get_attach_points(Color c) const
 {
     return m_attach_points[c];
@@ -514,11 +531,6 @@ inline BoardType Board::get_board_type() const
     return m_board_const->get_board_type();
 }
 
-inline unsigned Board::get_bonus(Color c) const
-{
-    return m_state_color[c].bonus;
-}
-
 inline const Geometry& Board::get_geometry() const
 {
     return *m_geo;
@@ -592,6 +604,11 @@ inline unsigned Board::get_nu_moves() const
     return m_moves.size();
 }
 
+inline Color::IntType Board::get_nu_nonalt_colors() const
+{
+    return m_variant != Variant::classic_3 ? m_nu_colors : 3;
+}
+
 inline unsigned Board::get_nu_onboard_pieces() const
 {
     return m_state_base.nu_onboard_pieces_all;
@@ -648,11 +665,6 @@ inline unsigned Board::get_points(Color c) const
     return m_state_color[c].points;
 }
 
-inline unsigned Board::get_points_with_bonus(Color c) const
-{
-    return get_points(c) + get_bonus(c);
-}
-
 inline Color Board::get_previous(Color c) const
 {
     return c.get_previous(m_nu_colors);
@@ -663,8 +675,8 @@ inline int Board::get_score(Color c) const
     if (m_nu_colors == 2)
     {
         LIBBOARDGAME_ASSERT(m_nu_players == 2);
-        unsigned points0 = get_points_with_bonus(Color(0));
-        unsigned points1 = get_points_with_bonus(Color(1));
+        unsigned points0 = get_points(Color(0));
+        unsigned points1 = get_points(Color(1));
         if (c == Color(0))
             return points0 - points1;
         else
@@ -673,10 +685,8 @@ inline int Board::get_score(Color c) const
     else if (m_nu_players == 2)
     {
         LIBBOARDGAME_ASSERT(m_nu_colors == 4);
-        unsigned points0 =
-            get_points_with_bonus(Color(0)) + get_points_with_bonus(Color(2));
-        unsigned points1 =
-            get_points_with_bonus(Color(1)) + get_points_with_bonus(Color(3));
+        unsigned points0 = get_points(Color(0)) + get_points(Color(2));
+        unsigned points1 = get_points(Color(1)) + get_points(Color(3));
         if (c == Color(0) || c == Color(2))
             return points0 - points1;
         else
@@ -684,14 +694,12 @@ inline int Board::get_score(Color c) const
     }
     else
     {
-        LIBBOARDGAME_ASSERT(m_nu_colors == m_nu_players);
         int score = 0;
-        for (ColorIterator i(m_nu_colors); i; ++i)
+        auto nu_players = static_cast<Color::IntType>(m_nu_players);
+        for (ColorIterator i(nu_players); i; ++i)
             if (*i != c)
-                score -= get_points_with_bonus(*i);
-        score =
-            get_points_with_bonus(c)
-            + score / (static_cast<int>(m_nu_colors) - 1);
+                score -= get_points(*i);
+        score = get_points(c) + score / (static_cast<int>(nu_players) - 1);
         return score;
     }
 }
@@ -860,9 +868,9 @@ inline void Board::place(Color c, Move mv)
         state_color.pieces_left.remove_fast(piece);
         if (state_color.pieces_left.empty())
         {
-            state_color.bonus = m_bonus_all_pieces;
+            state_color.points += m_bonus_all_pieces;
             if (piece_size == 1)
-                state_color.bonus += m_bonus_one_piece;
+                state_color.points += m_bonus_one_piece;
         }
     }
     ++m_state_base.nu_onboard_pieces_all;
@@ -950,7 +958,7 @@ inline void Board::restore_snapshot()
         state.forbidden.memcpy_from(snapshot_state.forbidden, *m_geo);
         state.is_attach_point.memcpy_from(snapshot_state.is_attach_point,
                                           *m_geo);
-        // Uncomment one is_trivially_copyable is implemented in GCC and MSVC
+        // Uncomment once is_trivially_copyable is implemented in GCC and MSVC
         //static_assert(is_trivially_copyable<StateColor>::value, "")
         memcpy(&state.pieces_left, &snapshot_state.pieces_left,
                sizeof(StateColor) - offsetof(StateColor, pieces_left));
@@ -972,7 +980,7 @@ inline string Board::to_string(Move mv, bool with_piece_name) const
 
 inline WritePoint Board::write(Point p) const
 {
-    return WritePoint(p, m_geo->get_width());
+    return WritePoint(p, m_geo->get_width(), m_geo->get_height());
 }
 
 //-----------------------------------------------------------------------------
