@@ -12,7 +12,7 @@
 
 #include "MoveList.h"
 #include "libboardgame_sgf/TreeReader.h"
-#include "libboardgame_sgf/Util.h"
+#include "libboardgame_sgf/SgfUtil.h"
 #include "libboardgame_util/Log.h"
 #include "libboardgame_util/RandomGenerator.h"
 
@@ -50,14 +50,10 @@ Engine::Engine(Variant variant)
     add("undo", &Engine::cmd_undo);
 }
 
-Engine::~Engine() throw()
-{
-}
-
 void Engine::board_changed()
 {
     if (m_show_board)
-        log() << get_board();
+        log(get_board());
 }
 
 void Engine::cmd_all_legal(const Arguments& args, Response& response)
@@ -81,7 +77,7 @@ void Engine::cmd_final_score(Response& response)
     if (get_nu_players(bd.get_variant()) > 2)
     {
         for (ColorIterator i(bd.get_nu_colors()); i; ++i)
-            response << bd.get_points_with_bonus(*i) << ' ';
+            response << bd.get_points(*i) << ' ';
     }
     else
     {
@@ -129,7 +125,7 @@ void Engine::cmd_loadsgf(const Arguments& args)
         reader.read(file);
         auto tree = reader.get_tree_transfer_ownership();
         m_game.init(tree);
-        const Node* node = nullptr;
+        const SgfNode* node = nullptr;
         if (move_number != -1)
             node = m_game.get_tree().get_node_before_move_number(move_number);
         if (node == nullptr)
@@ -233,8 +229,7 @@ void Engine::cmd_point_integers(Response& response)
     Grid<int> grid;
     for (GeometryIterator i(geo); i; ++i)
         grid[*i] = (*i).to_int();
-    response << '\n';
-    grid.write(response, geo);
+    response << '\n' << grid.to_string(geo);
 }
 
 void Engine::cmd_reg_genmove(const Arguments& args, Response& response)
@@ -296,7 +291,8 @@ void Engine::genmove(Color c, Response& response)
     }
     m_game.play(c, mv, true);
     response << bd.to_string(mv, false);
-    board_changed();
+    if (! mv.is_pass())
+        board_changed();
 }
 
 Color Engine::get_color_arg(const Arguments& args) const
@@ -311,10 +307,14 @@ Color Engine::get_color_arg(const Arguments& args, unsigned i) const
     string s = args.get_tolower(i);
     auto& bd = get_board();
     auto variant = bd.get_variant();
-    if (variant == Variant::classic || variant == Variant::classic_2
-        || variant == Variant::trigon || variant == Variant::trigon_2
-        || variant == Variant::trigon_3)
+    switch (variant)
     {
+    case Variant::classic:
+    case Variant::classic_2:
+    case Variant::classic_3:
+    case Variant::trigon:
+    case Variant::trigon_2:
+    case Variant::trigon_3:
         if (s == "1" || s == "blue")
             return Color(0);
         if (s == "2" || s == "yellow")
@@ -323,20 +323,19 @@ Color Engine::get_color_arg(const Arguments& args, unsigned i) const
             return Color(2);
         if (s == "4" || s == "green")
             return Color(3);
-    }
-    if (variant == Variant::duo || variant == Variant::junior)
-    {
+        break;
+    case Variant::duo:
+    case Variant::junior:
         if (s == "blue" || s == "black" || s == "b")
             return Color(0);
         if (s == "green" || s == "white" || s == "w")
             return Color(1);
+        break;
     }
-    ostringstream msg;
-    msg << "invalid color argument '" << s << "'";
-    throw Failure(msg.str());
+    throw Failure("invalid color argument '" + s + "'");
 }
 
-Player& Engine::get_player() const
+PlayerBase& Engine::get_player() const
 {
     if (m_player == nullptr)
         throw Failure("no player set");
@@ -363,10 +362,11 @@ void Engine::play(Color c, const Arguments& args, unsigned arg_move_begin)
     if (! m_accept_illegal && ! bd.is_legal(c, mv))
         throw Failure("illegal move");
     m_game.play(c, mv, true);
-    board_changed();
+    if (! mv.is_pass())
+        board_changed();
 }
 
-void Engine::set_player(Player& player)
+void Engine::set_player(PlayerBase& player)
 {
     m_player = &player;
     add("genmove", &Engine::cmd_genmove);
@@ -377,7 +377,7 @@ void Engine::set_player(Player& player)
 void Engine::set_show_board(bool enable)
 {
     if (enable && ! m_show_board)
-        log() << get_board();
+        log(get_board());
     m_show_board = enable;
 }
 
