@@ -25,11 +25,11 @@ using libpentobi_base::ColorIterator;
 ScoreDisplay::ScoreDisplay(QWidget* parent)
     : QWidget(parent),
       m_hasMoves(false),
-      m_points(0),
-      m_bonus(0)
+      m_points(0)
 {
     m_variant = Variant::classic;
-    m_font.setStyleStrategy(QFont::PreferOutline);
+    m_font.setStyleStrategy(QFont::StyleStrategy(QFont::PreferOutline
+                                                 | QFont::PreferQuality));
     setMinimumWidth(300);
     setMinimumHeight(20);
     setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
@@ -37,7 +37,7 @@ ScoreDisplay::ScoreDisplay(QWidget* parent)
 
 void ScoreDisplay::drawScore(QPainter& painter, Color c, int x)
 {
-    auto color = Util::getPaintColor(m_variant, c);
+    QColor color = Util::getPaintColor(m_variant, c);
     painter.setPen(Qt::NoPen);
     painter.setBrush(color);
     QFontMetrics metrics(m_font);
@@ -45,7 +45,8 @@ void ScoreDisplay::drawScore(QPainter& painter, Color c, int x)
     // y is baseline
     int y = static_cast<int>(ceil(0.5 * (height() - ascent)) + ascent);
     painter.setRenderHint(QPainter::Antialiasing, true);
-    painter.drawEllipse(x,  y - m_colorDotSize, m_colorDotSize, m_colorDotSize);
+    painter.drawEllipse(x,  y - m_colorDotSize, m_colorDotSize,
+                        m_colorDotSize);
     QString text = getScoreText(c);
     bool underline = ! m_hasMoves[c];
     drawText(painter, text, x + m_colorDotWidth, y, underline);
@@ -71,6 +72,33 @@ void ScoreDisplay::drawScore2(QPainter& painter, Color c1, Color c2, int x)
     drawText(painter, text, x + m_twoColorDotWidth, y, underline);
 }
 
+void ScoreDisplay::drawScore3(QPainter& painter, int x)
+{
+    auto color = Util::getPaintColor(m_variant, Color(3));
+    painter.setPen(Qt::NoPen);
+    painter.setBrush(color);
+    QFontMetrics metrics(m_font);
+    int ascent = metrics.ascent();
+    // y is baseline
+    int y = static_cast<int>(ceil(0.5 * (height() - ascent)) + ascent);
+    painter.setRenderHint(QPainter::Antialiasing, true);
+    if (m_hasMoves[Color(3)])
+    {
+        painter.drawEllipse(x, y - m_colorDotSize,
+                            m_colorDotSize, m_colorDotSize);
+        color = Util::getPaintColor(m_variant, m_altPlayer);
+        painter.setBrush(color);
+        painter.drawEllipse(x + m_colorDotSize, y - m_colorDotSize,
+                            m_colorDotSize, m_colorDotSize);
+    }
+    else
+        painter.drawEllipse(x + m_colorDotSize, y - m_colorDotSize,
+                            m_colorDotSize, m_colorDotSize);
+    QString text = getScoreText3();
+    bool underline = ! m_hasMoves[Color(3)];
+    drawText(painter, text, x + m_twoColorDotWidth, y, underline);
+}
+
 void ScoreDisplay::drawText(QPainter& painter, const QString& text, int x,
                             int y, bool underline)
 {
@@ -96,21 +124,26 @@ void ScoreDisplay::drawText(QPainter& painter, const QString& text, int x,
     }
 }
 
-QString ScoreDisplay::getScoreText(unsigned points, unsigned bonus) const
+QString ScoreDisplay::getScoreText(unsigned points) const
 {
     QString text;
-    text.setNum(points + bonus);
+    text.setNum(points);
     return text;
 }
 
 QString ScoreDisplay::getScoreText(Color c)
 {
-    return getScoreText(m_points[c], m_bonus[c]);
+    return getScoreText(m_points[c]);
 }
 
 QString ScoreDisplay::getScoreText2(Color c1, Color c2)
 {
-    return getScoreText(m_points[c1] + m_points[c2], m_bonus[c1] + m_bonus[c2]);
+    return getScoreText(m_points[c1] + m_points[c2]);
+}
+
+QString ScoreDisplay::getScoreText3()
+{
+    return "(" + getScoreText(Color(3)) + ")";
 }
 
 int ScoreDisplay::getScoreTextWidth(Color c)
@@ -123,6 +156,11 @@ int ScoreDisplay::getScoreTextWidth2(Color c1, Color c2)
     return getTextWidth(getScoreText2(c1, c2));
 }
 
+int ScoreDisplay::getScoreTextWidth3()
+{
+    return getTextWidth(getScoreText3());
+}
+
 int ScoreDisplay::getTextWidth(QString text) const
 {
     // Make text width only depend on number of digits to avoid frequent small
@@ -180,14 +218,33 @@ void ScoreDisplay::paintEvent(QPaintEvent*)
         x += m_colorDotWidth + textWidthRed + pad;
         drawScore(painter, Color(3), static_cast<int>(x));
     }
+    else if (m_variant == Variant::classic_3)
+    {
+        int textWidthBlue = getScoreTextWidth(Color(0));
+        int textWidthYellow = getScoreTextWidth(Color(1));
+        int textWidthRed = getScoreTextWidth(Color(2));
+        int textWidthGreen = getScoreTextWidth3();
+        int totalWidth =
+            textWidthBlue + textWidthRed + textWidthYellow + textWidthGreen
+            + 3 * m_colorDotWidth + m_twoColorDotWidth;
+        qreal pad = qreal(width() - totalWidth) / 5.f;
+        qreal x = pad;
+        drawScore(painter, Color(0), static_cast<int>(x));
+        x += m_colorDotWidth + textWidthBlue + pad;
+        drawScore(painter, Color(1), static_cast<int>(x));
+        x += m_colorDotWidth + textWidthYellow + pad;
+        drawScore(painter, Color(2), static_cast<int>(x));
+        x += m_colorDotWidth + textWidthRed + pad;
+        drawScore3(painter, static_cast<int>(x));
+    }
     else if (m_variant == Variant::trigon_3)
     {
         int textWidthBlue = getScoreTextWidth(Color(0));
         int textWidthYellow = getScoreTextWidth(Color(1));
         int textWidthRed = getScoreTextWidth(Color(2));
         int totalWidth =
-            textWidthBlue + textWidthRed + textWidthYellow
-            + 3 * m_colorDotWidth;
+                textWidthBlue + textWidthRed + textWidthYellow
+                + 3 * m_colorDotWidth;
         qreal pad = qreal(width() - totalWidth) / 4.f;
         qreal x = pad;
         drawScore(painter, Color(0), static_cast<int>(x));
@@ -240,17 +297,16 @@ void ScoreDisplay::updateScore(const Board& bd)
     for (ColorIterator i(bd.get_nu_colors()); i; ++i)
     {
         bool hasMoves = bd.has_moves(*i);
-        unsigned points = bd.get_points(*i);
-        unsigned bonus = bd.get_bonus(*i);
-        if (hasMoves != m_hasMoves[*i] || m_points[*i] != points
-            || m_bonus[*i] != bonus)
+        auto points = bd.get_points(*i);
+        if (hasMoves != m_hasMoves[*i] || m_points[*i] != points)
         {
             hasChanged = true;
             m_hasMoves[*i] = hasMoves;
             m_points[*i] = points;
-            m_bonus[*i] = bonus;
         }
     }
+    if (variant == Variant::classic_3)
+        m_altPlayer = Color(bd.get_alt_player());
     if (hasChanged)
         update();
 }
