@@ -10,24 +10,36 @@
 
 #include "State.h"
 
-#include "libboardgame_util/FmtSaver.h"
+#include "libboardgame_util/MathUtil.h"
 #include "libpentobi_base/BoardUtil.h"
+#include "libpentobi_base/ScoreUtil.h"
 
 namespace libpentobi_mcts {
 
 using namespace std;
-using libboardgame_util::FmtSaver;
+using libboardgame_util::fast_exp;
+using libpentobi_base::get_multiplayer_result;
 using libpentobi_base::BoardType;
 using libpentobi_base::ColorIterator;
 using libpentobi_base::PointState;
 
 //-----------------------------------------------------------------------------
 
+namespace {
+
+inline Float sigmoid(Float steepness, Float x)
+{
+    return -1.f + 2.f / (1.f + fast_exp(-steepness * x));
+}
+
+} // namespace
+
+//-----------------------------------------------------------------------------
+
 SharedConst::SharedConst(const Color& to_play)
     : board(nullptr),
       to_play(to_play),
-      avoid_symmetric_draw(true),
-      score_modification(0.1f)
+      avoid_symmetric_draw(true)
 {
     // Game variant and position dependent variables are initialized in
     // libpentobi_mcts::Search::start_search()
@@ -41,10 +53,6 @@ State::State(Variant initial_variant, const SharedConst& shared_const)
 {
 }
 
-State::~State() throw()
-{
-}
-
 inline void State::add_move(MoveList& moves, Move mv, double gamma)
 {
     m_total_gamma += gamma;
@@ -63,7 +71,7 @@ inline void State::add_moves(Point p, Color c,
     for (Piece piece : pieces_considered)
         for (Move mv : get_moves(c, piece, p, adj_status))
             if (! marker[mv]
-                && check_move(is_forbidden, get_move_info(mv), gamma))
+                    && check_move(is_forbidden, get_move_info(mv), gamma))
             {
                 marker.set(mv);
                 add_move(moves, mv, gamma);
@@ -102,7 +110,6 @@ void State::add_starting_moves(Color c,
     auto& is_forbidden = m_bd.is_forbidden(c);
     for (Piece piece : pieces_considered)
     {
-        auto piece_size = m_bd.get_piece_info(piece).get_size();
         for (Move mv : get_moves(c, piece, p, 0))
         {
             LIBBOARDGAME_ASSERT(! marker[mv]);
@@ -110,7 +117,7 @@ void State::add_starting_moves(Color c,
             {
                 marker.set(mv);
                 if (with_gamma)
-                    add_move(moves, mv, m_gamma_piece_size[piece_size]);
+                    add_move(moves, mv, m_gamma_piece[piece]);
                 else
                     moves.push_back(mv);
             }
@@ -127,15 +134,14 @@ bool State::check_move(const Grid<bool>& is_forbidden, const MoveInfo& info,
     if (is_forbidden[*i])
         return false;
     LocalValue::Compute local(*i, m_local_value);
-    unsigned piece_size = info.size();
-    auto end = i + piece_size;
+    auto end = info.end();
     while (++i != end)
     {
         if (is_forbidden[*i])
             return false;
         local.add_move_point(*i, m_local_value);
     }
-    gamma = m_gamma_piece_size[piece_size];
+    gamma = m_gamma_piece[info.get_piece()];
     if (local.has_local())
     {
         gamma *= m_gamma_nu_attach[local.get_nu_attach()];
@@ -158,113 +164,117 @@ bool State::check_move_without_gamma(const Grid<bool>& is_forbidden, Move mv)
     return true;
 }
 
-void State::dump(ostream& out) const
+#if LIBBOARDGAME_DEBUG
+string State::dump() const
 {
-    out << "pentobi_mcts::State:\n";
-    libpentobi_base::boardutil::dump(m_bd, out);
+    ostringstream s;
+    s << "pentobi_mcts::State:\n" << libpentobi_base::boardutil::dump(m_bd);
+    return s.str();
 }
+#endif
 
-array<Float,4> State::evaluate_playout()
+/** Evaluation function for game variants with 2 players and 1 color per
+    player. */
+void State::evaluate_duo(array<Float, 6>& result)
 {
-    // Always evaluate symmetric positions as a draw in the playouts. This
-    // will encourage the first player to break the symmetry and the second
-    // player to preserve it.
+    LIBBOARDGAME_ASSERT(m_bd.get_nu_players() == 2);
+    LIBBOARDGAME_ASSERT(m_bd.get_nu_colors() == 2);
     if (! m_is_symmetry_broken
-        && m_bd.get_nu_onboard_pieces() >= m_symmetry_min_nu_pieces)
+            && m_bd.get_nu_onboard_pieces() >= m_symmetry_min_nu_pieces)
     {
         if (log_simulations)
             log("Result: 0.5 (symmetry)");
-        array<Float,4> result;
-        for (ColorIterator i(m_nu_colors); i; ++i)
-            result[(*i).to_int()] = 0.5;
-        return result;
+        result[0] = result[1] = 0.5;
+        return;
     }
-
-    return evaluate_terminal();
+    Color c(0);
+    auto s = static_cast<Float>(m_bd.get_score(c));
+    Float res;
+    if (s > 0)
+        res = 1;
+    else if (s < 0)
+        res = 0;
+    else
+        res = 0.5;
+    if (log_simulations)
+        log("Result color ", c, ": sco=", s, " game_res=", res);
+    res += get_eval_bonus(c, res, s);
+    if (log_simulations)
+        log("res=", res);
+    result[0] = res;
+    result[1] = 1.f - res;
 }
 
-/** Get the game result for each color.
-    The result is 0,0.5,1 for loss/tie/win in 2-player variants. If there
-    are n &gt; 2 players, this is generalized in the following way: The scores
-    are sorted in ascending order. Each rank r_i (i in 0..n-1) is assigned
-    a result value of r_i/(n-1). If multiple players have the same score,
-    the result value is the average of all ranks with this score. So being
-    the single winner still gives the result 1 and having the lowest score
-    gives the result 0. Being the single winner is better than sharing the
-    best place, which is better than getting the second place, etc.
-
-    There is small modification applied to the result that encorages wins
-    with larger scores. */
-array<Float,4> State::evaluate_terminal()
+/** Evaluation function for game variants with 2 players and 2 colors per
+    player. */
+void State::evaluate_multicolor(array<Float, 6>& result)
 {
-    unsigned nu_players = m_bd.get_nu_players();
-    ColorMap<Float> points;
-    ColorMap<Float> score;
-    for (ColorIterator i(m_nu_colors); i; ++i)
-        points[*i] = Float(m_bd.get_points_with_bonus(*i));
+    LIBBOARDGAME_ASSERT(m_bd.get_nu_players() == 2);
+    LIBBOARDGAME_ASSERT(m_bd.get_nu_colors() == 4);
+    // Always evaluate symmetric positions in trigon_2 as a draw in the
+    // playouts. See comment in evaluate_playout_duo.
+    // m_is_symmetry_broken is always true in classic_2, no need to check for
+    // game variant.
+    if (! m_is_symmetry_broken
+            && m_bd.get_nu_onboard_pieces() >= m_symmetry_min_nu_pieces)
+    {
+        if (log_simulations)
+            log("Result: 0.5 (symmetry)");
+        result[0] = result[1] = result[2] = result[3] = 0.5;
+        return;
+    }
+
+    Color c(0);
+    auto s = static_cast<Float>(m_bd.get_score(c));
+    Float res;
+    if (s > 0)
+        res = 1;
+    else if (s < 0)
+        res = 0;
+    else
+        res = 0.5;
+    if (log_simulations)
+        log("Result color ", c, ": sco=", s, " game_res=", res);
+    res += get_eval_bonus(c, res, s);
+    if (log_simulations)
+        log("res=", res);
+    result[0] = result[2] = res;
+    result[1] = result[3] = 1.f - res;
+}
+
+/** Evaluation function for game variants with more than 2 players.
+    The result is 0,0.5,1 for loss/tie/win in 2-player variants. For n \> 2
+    players, this is generalized in the following way: The scores are sorted in
+    ascending order. Each rank r_i (i in 0..n-1) is assigned a result value of
+    r_i/(n-1). If multiple players have the same score, the result value is the
+    average of all ranks with this score. So being the single winner still
+    gives the result 1 and having the lowest score gives the result 0. Being
+    the single winner is better than sharing the best place, which is better
+    than getting the second place, etc. */
+void State::evaluate_multiplayer(array<Float, 6>& result)
+{
+    auto nu_players = m_bd.get_nu_players();
+    LIBBOARDGAME_ASSERT(nu_players > 2);
+    array<unsigned, Color::range> points;
     for (Color::IntType i = 0; i < nu_players; ++i)
-        score[Color(i)] = Float(m_bd.get_score(Color(i)));
-    if (m_nu_colors > nu_players)
-    {
-        LIBBOARDGAME_ASSERT(m_nu_colors == 4);
-        score[Color(2)] = score[Color(0)];
-        score[Color(3)] = score[Color(1)];
-    }
-    array<Float,Color::range> sorted_points;
-    if (nu_players > 2)
-    {
-        for (ColorIterator i(m_nu_colors); i; ++i)
-            sorted_points[(*i).to_int()] = points[*i];
-        sort(sorted_points.begin(), sorted_points.begin() + m_nu_colors);
-    }
-    array<Float,4> result_array;
+        points[i] = m_bd.get_points(Color(i));
+    array<Float, Color::range> game_result;
+    get_multiplayer_result(nu_players, points, game_result);
     for (Color::IntType i = 0; i < nu_players; ++i)
     {
         Color c(i);
-        Float game_result;
-        if (nu_players == 2)
-        {
-            if (score[c] > 0)
-                game_result = 1;
-            else if (score[c] < 0)
-                game_result = 0;
-            else
-                game_result = 0.5;
-        }
-        else
-        {
-            game_result = 0;
-            Float n = 0;
-            for (Color::IntType j = 0; j < m_nu_colors; ++j)
-                if (sorted_points[j] == points[c])
-                {
-                    game_result += Float(j) / Float(m_nu_colors - 1);
-                    ++n;
-                }
-            game_result /= n;
-        }
-
-        Float score_modification = m_shared_const.score_modification;
-        // Apply score modification. Example: If score modification is 0.1,
-        // the game result is rescaled to [0..0.9] and the score modification
-        // is added with 0.05 as the middle (corresponding to score 0).
-        Float result =
-            (1.f - score_modification) * game_result
-            + 0.5f * (score_modification
-                      + score[c] * m_score_modification_factor);
-        result_array[i] = result;
+        auto s = static_cast<Float>(m_bd.get_score(c));
+        result[i] = game_result[i] + get_eval_bonus(c, game_result[i], s);
         if (log_simulations)
-            log() << "Result color " << c << ": score=" << score[c]
-                  << " game_result=" << game_result << " result=" << result
-                  << '\n';
+            log("Result color ", c, ": sco=", s, " game_res=", game_result[i],
+                " res=", result[i]);
     }
-    if (m_nu_colors > nu_players)
+    if (m_bd.get_variant() == Variant::classic_3)
     {
-        LIBBOARDGAME_ASSERT(m_nu_colors == 4);
-        result_array[2] = result_array[0];
-        result_array[3] = result_array[1];
+        result[3] = result[0];
+        result[4] = result[1];
+        result[5] = result[2];
     }
-    return result_array;
 }
 
 Point State::find_best_starting_point(Color c) const
@@ -312,13 +322,13 @@ Point State::find_best_starting_point(Color c) const
     return best;
 }
 
-bool State::gen_playout_move(Move lgr1, Move lgr2, Move& mv)
+bool State::gen_playout_move(Move lgr1, Move lgr2, PlayerMove<Move>& mv)
 {
     if (m_nu_passes == m_nu_colors)
         return false;
 
     if (! m_is_symmetry_broken
-        && m_bd.get_nu_onboard_pieces() >= m_symmetry_min_nu_pieces)
+            && m_bd.get_nu_onboard_pieces() >= m_symmetry_min_nu_pieces)
     {
         // See also the comment in evaluate_playout()
         if (log_simulations)
@@ -326,35 +336,31 @@ bool State::gen_playout_move(Move lgr1, Move lgr2, Move& mv)
         return false;
     }
 
-    ++m_nu_playout_moves;
+    Color to_play = m_bd.get_to_play();
     if (lgr2.is_regular() && m_bd.is_legal_nonpass(lgr2))
     {
         if (log_simulations)
             log("Playing last good reply 2");
-        ++m_nu_last_good_reply_moves;
-        mv = lgr2;
+        mv = PlayerMove<Move>(to_play.to_int(), lgr2);
         return true;
     }
     if (lgr1.is_regular() && m_bd.is_legal_nonpass(lgr1))
     {
         if (log_simulations)
             log("Playing last good reply 1");
-        ++m_nu_last_good_reply_moves;
-        mv = lgr1;
+        mv = PlayerMove<Move>(to_play.to_int(), lgr1);
         return true;
     }
 
-    Color to_play;
     while (true)
     {
-        to_play = m_bd.get_to_play();
         if (! m_is_move_list_initialized[to_play])
             init_moves_with_gamma(to_play);
         else if (m_has_moves[to_play])
             update_moves(to_play);
         if ((m_has_moves[to_play] = ! m_moves[to_play].empty()))
             break;
-        if (m_nu_passes + 1 == m_nu_colors)
+        if (++m_nu_passes == m_nu_colors)
             return false;
         if (m_check_terminate_early && m_bd.get_score(to_play) < 0
             && ! m_has_moves[m_bd.get_second_color(to_play)])
@@ -363,34 +369,45 @@ bool State::gen_playout_move(Move lgr1, Move lgr2, Move& mv)
                 log("Terminate early (no moves and negative score)");
             return false;
         }
-        ++m_nu_passes;
-        m_bd.set_to_play(m_bd.get_next(to_play));
+        to_play = to_play.get_next(m_nu_colors);
+        m_bd.set_to_play(to_play);
         // Don't try to handle symmetry after pass moves
         m_is_symmetry_broken = true;
     }
 
     auto& moves = m_moves[to_play];
     if (log_simulations)
-        log() << "Moves: " << moves.size() << ", total gamma: "
-              << m_total_gamma << "\n";
+        log("Moves: ", moves.size(), ", total gamma: ", m_total_gamma);
     auto begin = m_cumulative_gamma.begin();
     auto end = begin + moves.size();
     auto random = m_total_gamma * m_random.generate_double();
     auto pos = lower_bound(begin, end, random);
     LIBBOARDGAME_ASSERT(pos != end);
-    mv = moves[static_cast<unsigned>(pos - begin)];
+    mv = PlayerMove<Move>(to_play.to_int(),
+                          moves[static_cast<unsigned>(pos - begin)]);
     return true;
 }
 
-bool State::gen_and_play_playout_move(Move lgr1, Move lgr2)
+/** Bonus added to the result to encorage wins with larger scores and
+    fewer number of moves.
+    See also: Pepels et al.: Quality-based Rewards for Monte-Carlo Tree Search
+    Simulations. ECAI 2014. */
+inline Float State::get_eval_bonus(Color c, Float result, Float score)
 {
-    Move mv;
-    if (gen_playout_move(lgr1, lgr2, mv))
-    {
-        play_playout(mv);
-        return true;
-    }
-    return false;
+    Float bonus = 0;
+    Float l = static_cast<Float>(m_bd.get_nu_moves());
+    m_stat_len.add(l);
+    Float dev = m_stat_len.get_deviation();
+    if (dev > 0)
+        bonus +=
+                (result == 1 ? -0.06f : 0.06f)
+                * sigmoid(2.f, (l - m_stat_len.get_mean()) / dev);
+    auto& stat = m_stat_score[c];
+    stat.add(score);
+    dev = stat.get_deviation();
+    if (dev > 0)
+        bonus += 0.3f * sigmoid(2.f, (score - stat.get_mean()) / dev);
+    return bonus;
 }
 
 inline const PieceMap<bool>& State::get_pieces_considered() const
@@ -412,7 +429,7 @@ void State::init_moves_with_gamma(Color c)
     m_total_gamma = 0;
     auto& marker = m_marker[c];
     auto& moves = m_moves[c];
-    marker.clear_all_set_known(moves);
+    marker.clear(moves);
     moves.clear();
     Board::PiecesLeftList pieces_considered;
     for (Piece piece : m_bd.get_pieces_left(c))
@@ -438,7 +455,7 @@ void State::init_moves_without_gamma(Color c)
     m_is_piece_considered[c] = &get_pieces_considered();
     auto& marker = m_marker[c];
     auto& moves = m_moves[c];
-    marker.clear_all_set_known(moves);
+    marker.clear(moves);
     moves.clear();
     Board::PiecesLeftList pieces_considered;
     for (Piece piece : m_bd.get_pieces_left(c))
@@ -455,7 +472,7 @@ void State::init_moves_without_gamma(Color c)
                 for (Piece piece : pieces_considered)
                     for (Move mv : get_moves(c, piece, p, adj_status))
                         if (! marker[mv]
-                            && check_move_without_gamma(is_forbidden, mv))
+                                && check_move_without_gamma(is_forbidden, mv))
                         {
                             marker.set(mv);
                             moves.push_back(mv);
@@ -479,30 +496,18 @@ void State::play_expanded_child(Move mv)
         play_playout(mv);
     else
     {
-        m_bd.play_pass();
         ++m_nu_passes;
+        m_bd.set_to_play(m_bd.get_to_play().get_next(m_nu_colors));
         // Don't try to handle pass moves: a pass move either breaks symmetry
         // or both players have passed and it's the end of the game and we need
-        // symmetry detection only as a heuristic ((playouts and move value
+        // symmetry detection only as a heuristic (playouts and move value
         // initialization)
         m_is_symmetry_broken = true;
         if (log_simulations)
-            log() << m_bd;
+            log(m_bd);
     }
 }
 
-void State::play_playout(Move mv)
-{
-    LIBBOARDGAME_ASSERT(m_bd.is_legal_nonpass(mv));
-    m_new_moves[m_bd.get_to_play()].push_back(mv);
-    m_bd.play_nonpass(mv);
-    m_nu_passes = 0;
-    if (! m_is_symmetry_broken)
-        update_symmetry_broken(mv);
-    if (log_simulations)
-        log() << m_bd;
-}
-
 void State::start_search()
 {
     auto& bd = *m_shared_const.board;
@@ -513,16 +518,9 @@ void State::start_search()
     m_nu_colors = bd.get_nu_colors();
     m_move_info_array = m_bc->get_move_info_array();
     m_move_info_ext_array = m_bc->get_move_info_ext_array();
-    m_nu_moves_initial = bd.get_nu_moves();
     m_check_terminate_early =
-        (m_nu_moves_initial < 10u * m_nu_colors
-         && m_bd.get_nu_players() == 2);
-    Float total_piece_points = Float(m_bc->get_total_piece_points());
-    m_score_modification_factor =
-        m_shared_const.score_modification / total_piece_points;
-    m_nu_simulations = 0;
-    m_nu_playout_moves = 0;
-    m_nu_last_good_reply_moves = 0;
+            (bd.get_nu_moves() < 10u * m_nu_colors
+             && m_bd.get_nu_players() == 2);
     auto variant = bd.get_variant();
     m_check_symmetric_draw =
         ((variant == Variant::duo || variant == Variant::junior
@@ -531,31 +529,58 @@ void State::start_search()
                 || m_shared_const.to_play == Color(3))
                && m_shared_const.avoid_symmetric_draw)
          && ! check_symmetry_broken(bd, m_shared_const.symmetric_points));
+    if (! m_check_symmetric_draw)
+        // Pretending that the symmetry is always broken is equivalent to
+        // ignoring symmetric draws
+        m_is_symmetry_broken = true;
     if (variant == Variant::trigon_2)
         m_symmetry_min_nu_pieces = 5;
     else
         m_symmetry_min_nu_pieces = 3; // Only used in Duo
 
     m_prior_knowledge.start_search(bd);
+    m_stat_len.clear();
+    for (ColorIterator i(m_nu_colors); i; ++i)
+        m_stat_score[*i].clear();
 
     // Init precomputed gamma values
-    double gamma_size_factor = bd.get_board_type() == BoardType::duo ? 3 : 5;
-    double gamma = 1;
-    for (unsigned i = 1; i < PieceInfo::max_size + 1;
-         ++i, gamma *= gamma_size_factor)
-        m_gamma_piece_size[i] = gamma;
-    gamma = 1e10;
-    for (unsigned i = 0; i < PieceInfo::max_size + 1; ++i, gamma *= 1e10)
-        m_gamma_nu_attach[i] = gamma;
+    double gamma_size_factor = 1;
+    double gamma_nu_attach_factor = 1;
+    switch (bd.get_board_type())
+    {
+    case BoardType::classic:
+        gamma_size_factor = 5;
+        break;
+    case BoardType::duo:
+        gamma_size_factor = 3;
+        gamma_nu_attach_factor = 1.8;
+        break;
+    case BoardType::trigon:
+    case BoardType::trigon_3:
+        gamma_size_factor = 5;
+        break;
+    }
+    for (Piece::IntType i = 0; i < m_bc->get_nu_pieces(); ++i)
+    {
+        Piece piece(i);
+        auto piece_size = m_bc->get_piece_info(piece).get_size();
+        LIBBOARDGAME_ASSERT(piece_size > 0);
+        auto piece_nu_attach = m_bc->get_nu_attach_points(piece);
+        LIBBOARDGAME_ASSERT(piece_nu_attach > 0);
+        m_gamma_piece[piece] =
+            pow(gamma_size_factor, piece_size - 1)
+            * pow(gamma_nu_attach_factor, piece_nu_attach - 1);
+    }
+    for (unsigned i = 0; i < PieceInfo::max_size + 1; ++i)
+        m_gamma_nu_attach[i] = pow(1e10, i);
 }
 
 void State::start_simulation(size_t n)
 {
     if (log_simulations)
-        log() << "=========================================================\n"
-              << "Simulation " << n << "\n"
-              << "=========================================================\n";
-    ++m_nu_simulations;
+        log("=========================================================\n",
+            "Simulation ", n, "\n",
+            "=========================================================");
     m_bd.restore_snapshot();
     m_force_consider_all_pieces = false;
     for (ColorIterator i(m_nu_colors); i; ++i)
@@ -583,24 +608,43 @@ void State::update_moves(Color c)
     auto& marker = m_marker[c];
 
     // Find old moves that are still legal
-    PieceMap<bool> is_piece_left(false);
-    for (Piece piece : m_bd.get_pieces_left(c))
-        is_piece_left[piece] = true;
     auto& is_forbidden = m_bd.is_forbidden(c);
+    auto& new_moves = m_new_moves[c];
     auto& moves = m_moves[c];
     auto old_size = moves.size();
     moves.clear();
     double gamma;
-    for (unsigned i = 0; i < old_size; ++i)
+    if (new_moves.size() == 1 && m_bd.get_nu_piece_instances() == 1)
     {
-        LIBBOARDGAME_ASSERT(i >= moves.size());
-        Move mv = moves.get_unchecked(i);
-        auto& info = get_move_info(mv);
-        if (is_piece_left[info.get_piece()]
-            && check_move(is_forbidden, info, gamma))
-            add_move(moves, mv, gamma);
-        else
-            marker.clear(mv);
+        Piece piece = get_move_info(new_moves[0]).get_piece();
+        for (unsigned i = 0; i < old_size; ++i)
+        {
+            LIBBOARDGAME_ASSERT(i >= moves.size());
+            Move mv = moves.get_unchecked(i);
+            auto& info = get_move_info(mv);
+            if (info.get_piece() != piece
+                    && check_move(is_forbidden, info, gamma))
+                add_move(moves, mv, gamma);
+            else
+                marker.clear(mv);
+        }
+    }
+    else
+    {
+        PieceMap<bool> is_piece_left(false);
+        for (Piece piece : m_bd.get_pieces_left(c))
+            is_piece_left[piece] = true;
+        for (unsigned i = 0; i < old_size; ++i)
+        {
+            LIBBOARDGAME_ASSERT(i >= moves.size());
+            Move mv = moves.get_unchecked(i);
+            auto& info = get_move_info(mv);
+            if (is_piece_left[info.get_piece()]
+                    && check_move(is_forbidden, info, gamma))
+                add_move(moves, mv, gamma);
+            else
+                marker.clear(mv);
+        }
     }
 
     // Find new legal moves because of new pieces played by this color
@@ -608,7 +652,7 @@ void State::update_moves(Color c)
     for (Piece piece : m_bd.get_pieces_left(c))
         if ((*m_is_piece_considered[c])[piece])
             pieces_considered.push_back(piece);
-    for (Move mv : m_new_moves[c])
+    for (Move mv : new_moves)
     {
         auto& info_ext = get_move_info_ext(mv);
         auto i = info_ext.begin_attach();
@@ -618,7 +662,7 @@ void State::update_moves(Color c)
                 add_moves(*i, c, pieces_considered);
         while (++i != end);
     }
-    m_new_moves[c].clear();
+    new_moves.clear();
 
     // Generate moves for pieces not considered in the last position
     auto& is_piece_considered = *m_is_piece_considered[c];
@@ -649,14 +693,16 @@ void State::update_moves(Color c)
 void State::update_symmetry_broken(Move mv)
 {
     LIBBOARDGAME_ASSERT(! mv.is_pass());
-    auto& info = get_move_info(mv);
     Color to_play = m_bd.get_to_play();
     Color second_color = m_bd.get_second_color(to_play);
+    auto& info = get_move_info(mv);
+    auto i = info.begin();
+    auto end = info.end();
     if (to_play == Color(0) || to_play == Color(2))
     {
         // First player to play: Check that all symmetric points of the last
         // move of the second player are occupied by the first player
-        for (auto i = info.begin(); i != info.end(); ++i)
+        do
         {
             Point symm_p = m_shared_const.symmetric_points[*i];
             if (m_bd.get_point_state(symm_p) != second_color)
@@ -665,13 +711,14 @@ void State::update_symmetry_broken(Move mv)
                 return;
             }
         }
+        while (++i != end);
     }
     else
     {
         // Second player to play: Check that all symmetric points of the last
         // move of the first player are empty (i.e. the second player can play
         // there to preserve the symmetry)
-        for (auto i = info.begin(); i != info.end(); ++i)
+        do
         {
             Point symm_p = m_shared_const.symmetric_points[*i];
             if (! m_bd.get_point_state(symm_p).is_empty())
@@ -680,20 +727,20 @@ void State::update_symmetry_broken(Move mv)
                 return;
             }
         }
+        while (++i != end);
     }
 }
 
-void State::write_info(ostream& out) const
+string State::get_info() const
 {
-    if (m_nu_playout_moves > 0)
+    ostringstream s;
+    if (m_bd.get_nu_players() == 2)
     {
-        FmtSaver saver(log());
-        out << "LGR: " << fixed << setprecision(1)
-            << (100.0 * static_cast<double>(m_nu_last_good_reply_moves)
-                / static_cast<double>(m_nu_playout_moves))
-            << "%";
+        s << "Sco: ";
+        m_stat_score[Color(0)].write(s, true, 1);
     }
-    out << '\n';
+    s << '\n';
+    return s.str();
 }
 
 //-----------------------------------------------------------------------------
