@@ -15,7 +15,7 @@
 #include <QProgressDialog>
 #include <QtConcurrentRun>
 #include "Util.h"
-#include "libboardgame_sgf/Util.h"
+#include "libboardgame_sgf/SgfUtil.h"
 #include "libboardgame_util/Abort.h"
 #include "libboardgame_util/Log.h"
 #include "libpentobi_gui/Util.h"
@@ -26,7 +26,7 @@ using libboardgame_util::log;
 using libboardgame_util::set_abort;
 using libboardgame_util::ArrayList;
 using libpentobi_base::Board;
-using libpentobi_base::Tree;
+using libpentobi_base::PentobiTree;
 
 //-----------------------------------------------------------------------------
 
@@ -124,6 +124,11 @@ void AnalyzeGameWidget::paintEvent(QPaintEvent*)
         if (! m_analyzeGame.has_value(i))
             continue;
         double value = m_analyzeGame.get_value(i);
+        // Values can be outside [0..1] due to score/length bonuses
+        if (value < 0)
+            value = 0;
+        else if (value > 1)
+            value = 1;
         auto color = Util::getPaintColor(m_analyzeGame.get_variant(),
                                          m_analyzeGame.get_move(i).color);
         painter.setPen(Qt::NoPen);
@@ -140,7 +145,8 @@ void AnalyzeGameWidget::resizeEvent(QResizeEvent*)
     initSize();
 }
 
-void AnalyzeGameWidget::setCurrentPosition(const Game& game, const Node& node)
+void AnalyzeGameWidget::setCurrentPosition(const Game& game,
+                                           const SgfNode& node)
 {
     update();
     m_currentPosition = -1;
