@@ -36,7 +36,7 @@ class RatingDialog;
 class ScoreDisplay;
 
 using namespace std;
-using libboardgame_sgf::Node;
+using libboardgame_sgf::SgfNode;
 using libboardgame_base::Transform;
 using libboardgame_util::ArrayList;
 using libboardgame_util::Exception;
@@ -63,6 +63,8 @@ public:
                const QString& booksDir = "", bool noBook = false,
                unsigned nu_threads = 0, size_t memory = 0);
 
+    ~MainWindow();
+
     bool eventFilter(QObject* object, QEvent* event);
 
     QSize sizeHint() const;
@@ -252,7 +254,7 @@ private:
     };
 
     /** Possible values for m_level are in 1..maxLevel */
-    static const int maxLevel = 8;
+    static const int maxLevel = 9;
 
     static const int maxRecentFiles = 9;
 
@@ -285,8 +287,6 @@ private:
         we only need to handle edits. */
     bool m_ignoreCommentTextChanged;
 
-    bool m_genMoveInterrupted;
-
     /** Color played by the user in a rated game.
         Only defined if m_isRated is true. In game variants with multiple
         colors per player, the user plays all colors of the player with
@@ -400,6 +400,8 @@ private:
 
     QAction* m_actionVariantClassic2;
 
+    QAction* m_actionVariantClassic3;
+
     QAction* m_actionVariantDuo;
 
     QAction* m_actionVariantJunior;
@@ -578,8 +580,6 @@ private:
 
     QFutureWatcher<GenMoveResult> m_genMoveWatcher;
 
-    QElapsedTimer m_genMoveTime;
-
     QString m_file;
 
     unique_ptr<ArrayList<Move, Move::range>> m_legalMoves;
@@ -657,7 +657,7 @@ private:
 
     QString getVersion() const;
 
-    void gotoNode(const Node& node);
+    void gotoNode(const SgfNode& node);
 
     void initGame();
 
@@ -665,10 +665,14 @@ private:
 
     void initPieceSelectors();
 
+    bool isComputerToPlay() const;
+
     void leaveSetupMode();
 
     void play(Color c, Move mv);
 
+    void restoreLevel(Variant variant);
+
     bool save(const QString& file);
 
     void searchCallback(double elapsedSeconds, double remainingSeconds);
@@ -718,6 +722,8 @@ private slots:
 
     void commentChanged();
 
+    void continueRatedGame();
+
     void coordinates(bool checked);
 
     void doubtfulMove(bool checked);
@@ -728,6 +734,8 @@ private slots:
 
     void variantClassic2(bool checked);
 
+    void variantClassic3(bool checked);
+
     void variantDuo(bool checked);
 
     void variantJunior(bool checked);
