@@ -17,11 +17,11 @@
 #include <QSettings>
 #include <QString>
 #include "Util.h"
-#include "libpentobi_base/TreeWriter.h"
+#include "libpentobi_base/PentobiTreeWriter.h"
 #include "libpentobi_mcts/Player.h"
 
 using libpentobi_base::to_string_id;
-using libpentobi_base::TreeWriter;
+using libpentobi_base::PentobiTreeWriter;
 using libpentobi_mcts::Player;
 
 //-----------------------------------------------------------------------------
@@ -29,7 +29,7 @@ using libpentobi_mcts::Player;
 namespace {
 
 /** 1000 Elo represents a beginner level. */
-const double startRating = 1000;
+const float startRating = 1000;
 
 QString getRatedGamesDir(Variant variant)
 {
@@ -49,9 +49,10 @@ RatingHistory::RatingHistory(Variant variant)
 void RatingHistory::addGame(float score, Rating opponentRating,
                             unsigned nuOpponents, Color color,
                             float result, const QString& date, int level,
-                            const Tree& tree)
+                            const PentobiTree& tree)
 {
-    m_rating.update_multiplayer(score, opponentRating, nuOpponents, 32);
+    float k_value = (m_nuGames < 30 ? 40.f : 20.f);
+    m_rating.update_multiplayer(score, opponentRating, nuOpponents, k_value);
     if (m_rating.get() > m_bestRating.get())
         m_bestRating = m_rating;
     ++m_nuGames;
@@ -68,10 +69,8 @@ void RatingHistory::addGame(float score, Rating opponentRating,
         m_games.erase(m_games.begin(), m_games.begin() + nuGames - maxGames);
     save();
     ofstream out(getFile(m_nuGames).toLocal8Bit().constData());
-    TreeWriter writer(out, tree);
-    writer.set_one_prop_per_line(true);
-    writer.set_one_prop_value_per_line(true);
-    writer.set_indent(2);
+    PentobiTreeWriter writer(out, tree);
+    writer.set_indent(1);
     writer.write();
     // Only save the last RatingHistory::maxGames games
     if (m_nuGames > maxGames)
