@@ -12,13 +12,13 @@
 
 #include <fstream>
 #include "libboardgame_sgf/TreeReader.h"
-#include "libpentobi_base/Tree.h"
 #include "libpentobi_mcts/Util.h"
 
 namespace pentobi_gtp {
 
 using libboardgame_gtp::Failure;
 using libboardgame_mcts::ChildIterator;
+using libboardgame_mcts::PlayerMove;
 using libboardgame_sgf::TreeReader;
 using libboardgame_util::log;
 using libboardgame_util::ArrayList;
@@ -27,10 +27,8 @@ using libpentobi_base::Color;
 using libpentobi_base::Grid;
 using libpentobi_base::Move;
 using libpentobi_base::MoveList;
-using libpentobi_base::MoveMarker;
 using libpentobi_base::PieceInfo;
 using libpentobi_base::Point;
-using libpentobi_base::Tree;
 using libpentobi_mcts::Float;
 using libpentobi_mcts::Move;
 using libpentobi_mcts::State;
@@ -49,12 +47,11 @@ Engine::Engine(Variant variant, int level, bool use_book,
     add("name", &Engine::cmd_name);
     add("param", &Engine::cmd_param);
     add("move_values", &Engine::cmd_move_values);
-    add("moves_stat", &Engine::cmd_moves_stat);
     add("save_tree", &Engine::cmd_save_tree);
     add("version", &Engine::cmd_version);
 }
 
-Engine::~Engine() throw()
+Engine::~Engine()
 {
 }
 
@@ -80,28 +77,6 @@ void Engine::cmd_move_values(Response& response)
                  << bd.to_string(node->get_move()) << '\n';
 }
 
-void Engine::cmd_moves_stat(const Arguments& args, Response& response)
-{
-    Color c = get_color_arg(args);
-    auto& bd = get_board();
-    auto& geo = bd.get_geometry();
-    Grid<unsigned> nu_moves_grid;
-    nu_moves_grid.fill(0, geo);
-    MoveList moves;
-    MoveMarker marker;
-    for (Point p : bd.get_attach_points(c))
-    {
-        bd.gen_moves(c, p, marker, moves);
-        for (Move mv : moves)
-            for (Point p : bd.get_move_info(mv))
-                ++nu_moves_grid[p];
-        marker.clear_all_set_known(moves);
-        moves.clear();
-    }
-    response << '\n';
-    nu_moves_grid.write(response, geo);
-}
-
 void Engine::cmd_name(Response& response)
 {
     response.set("Pentobi");
@@ -125,7 +100,6 @@ void Engine::cmd_param(const Arguments& args, Response& response)
             << "avoid_symmetric_draw " << s.get_avoid_symmetric_draw() << '\n'
             << "auto_param " << s.get_auto_param() << '\n'
             << "bias_term_constant " << s.get_bias_term_constant() << '\n'
-            << "bias_term_interval " << s.get_bias_term_interval() << '\n'
             << "expand_threshold " << s.get_expand_threshold() << '\n'
             << "expand_threshold_inc " << s.get_expand_threshold_inc() << '\n'
             << "fixed_simulations " << p.get_fixed_simulations() << '\n'
@@ -135,8 +109,6 @@ void Engine::cmd_param(const Arguments& args, Response& response)
             << "rave_parent_max " << s.get_rave_parent_max() << '\n'
             << "rave_weight " << s.get_rave_weight() << '\n'
             << "reuse_subtree " << s.get_reuse_subtree() << '\n'
-            << "score_modification " << s.get_score_modification() << '\n'
-            << "skip_bias_term_min " << s.get_skip_bias_term_min() << '\n'
             << "use_book " << p.get_use_book() << '\n';
     else
     {
@@ -148,8 +120,6 @@ void Engine::cmd_param(const Arguments& args, Response& response)
             s.set_auto_param(args.parse<bool>(1));
         else if (name == "bias_term_constant")
             s.set_bias_term_constant(args.parse<Float>(1));
-        else if (name == "bias_term_interval")
-            s.set_bias_term_interval(args.parse<unsigned>(1));
         else if (name == "expand_threshold")
             s.set_expand_threshold(args.parse<Float>(1));
         else if (name == "expand_threshold_inc")
@@ -168,10 +138,6 @@ void Engine::cmd_param(const Arguments& args, Response& response)
             s.set_rave_weight(args.parse<Float>(1));
         else if (name == "reuse_subtree")
             s.set_reuse_subtree(args.parse<bool>(1));
-        else if (name == "score_modification")
-            s.set_score_modification(args.parse<Float>(1));
-        else if (name == "skip_bias_term_min")
-            s.set_skip_bias_term_min(args.parse<Float>(1));
         else if (name == "use_book")
             p.set_use_book(args.parse<bool>(1));
         else
@@ -219,10 +185,11 @@ void Engine::cmd_gen_playout_move(Response& response)
     state.start_search();
     state.start_simulation(0);
     state.finish_in_tree();
-    if (! state.gen_and_play_playout_move(Move::null(), Move::null()))
+    PlayerMove<Move> mv;
+    if (! state.gen_playout_move(Move::null(), Move::null(), mv))
         throw Failure("terminal playout position");
     auto& bd = get_board();
-    response << bd.to_string(state.get_move(state.get_nu_moves() - 1).move);
+    response << bd.to_string(mv.move);
 }
 
 Player& Engine::get_mcts_player()
